/*****************************************************************************
  jvofendi - a simple program to split lujvo into component rafsi and gismu

  Copyright Martin Bays 2003

  Main code adapted from some from 'jbofihe', by Richard P. Curnow,
  auxiliary code taken from 'valfendi', by Pierre Abbot.

  Last changes: Fri Oct  3 21:24:03 GMT 2003
  Contact: mbays@freeshell.org
  Version: 0.1 (initial release)


  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.  
*****************************************************************************/

#include <stdio.h>
#include <string.h>

char *readline();

static void append_terfendi(char *w, char *result, int initial);

static char *translate_lujvo(char *word);

int main(int argc, char **argv)
{
    if (argc > 1) {
	int i;
	for (i = 1; i < argc; i++)
	    printf("%s\n", translate_lujvo(argv[1]));
    }
    else 
	do {
	    printf("%s\n", translate_lujvo(readline()));
	} while (!feof(stdin));

    return 0;
}

char *readline()
    /* Reads one line of arbitrary length.
       Taken from Pierre Abbat's valfendi */
{char *line;
    int length;
    int ch;
    line=malloc(1);
    line[0]=0;
    length=0;
    do {ch=getchar();
	if (ch>31)
	{line[length]=ch;
	    line=realloc(line,(++length)+1);
	    line[length]=0;
	    if (!line) /* realloc failed */
	    {fputs("dukse ke seltcidu lijvelsku mi'e jvofendi\n",stderr);
		exit(1);
	    }
	}
    }
    while (ch!='\n' && ch>=0);
    return line;
}



    static void
append_terfendi(char *w, char *result, int initial)
{
    if (initial) {
	strcpy(result, w);
    } else {
	strcat(result, " ");
	strcat(result, w);
    }
}

    static char *
translate_lujvo(char *word)
{
    static char result[4096];
    char buf[64];
    char *w, *trans;
    int initial;
    char *yy;
    int ypos=0;
    static char terfendi[10][6];

    w = word;
    result[0] = 0;
    initial = 1;

    do {
	if (strlen(w) == 5) {
	    /* trailing gismu */
	    append_terfendi(w, result, initial);
	    break;
	} else {
	    /* Not 5 letter, see what else */
	    if (strlen(w) < 5) {
		/* Don't bother about whether this is initial or not, we need
		   to be able to lookup
		   single rafsi for breaking fuivla up. */

		strcpy(buf,"%");
		strcat(buf,w);
		append_terfendi(w, result, initial);
		break;
	    } else {
		/* Try to pull off leading rafsi component and process
		   remainder */
		yy = strchr(w, 'y');
		if (yy) {
		    ypos = yy - w;
		}
		if (yy && ypos == 3) {
		    /* 3 letter rafsi with y after */
		    strncpy(buf,w,3);
		    buf[3] = '\0';
		    append_terfendi(buf, result, initial);
		    w += 4; /* and go around again */
		} else if (yy && ypos == 4) {
		    /* 4 letter rafsi with y after */
		    strncpy(buf,w,4);
		    buf[4] = '\0';
		    append_terfendi(buf, result, initial);
		    w += 5; /* and go around again */

		} else {
		    /* 'y' does not terminate leading rafsi, or there is no 'y'
		       in the word at all.  Try to pull off 3 letter rafsi, or 4
		       letter one of form CV'V.  Remember to remove following
		       glue character if necessary, can only apply to first
		       rafsi. */
		    if (w[2] == '\'') {
			strncpy(buf,w,4);
			buf[4] = '\0';
			append_terfendi(buf, result, initial);

			if (strchr("aeiou", w[1]) &&
				strchr("aeiou", w[3]) &&
				initial &&
				w[4] &&
				(w[4] == 'r' ||
				 (w[4] == 'n' && w[5] == 'r'))) {
			    w += 5;
			} else {
			    w += 4;
			}
		    } else {
			strncpy(buf,w,3);
			buf[3] = '\0';
			append_terfendi(buf, result, initial);
			if (strchr("aeiou", w[1]) &&
				strchr("aeiou", w[2]) &&
				initial &&
				w[3] &&
				(w[3] == 'r' ||
				 (w[3] == 'n' && w[4] == 'r'))) {
			    w += 4;
			} else {
			    w += 3;
			}
		    }          
		}
	    }
	}

	initial = 0;

    } while (*w);
    return result;
}
